@@ -5,17 +5,17 @@ from itertools import count
 from zope.interface import implements
 from twisted.internet import defer
 from twisted.python import failure
-from allmydata.interfaces import IPublishStatus, FileTooLargeError
+from allmydata.interfaces import IPublishStatus
 from allmydata.util import base32, hashutil, mathutil, idlib, log
 from allmydata import hashtree, codec
 from allmydata.storage.server import si_b2a
 from pycryptopp.cipher.aes import AES
-from foolscap.eventual import eventually
+from foolscap.api import eventually, fireEventually
 
-from common import MODE_WRITE, MODE_CHECK, DictOfSets, \
+from allmydata.mutable.common import MODE_WRITE, MODE_CHECK, DictOfSets, \
      UncoordinatedWriteError, NotEnoughServersError
-from servermap import ServerMap
-from layout import pack_prefix, pack_share, unpack_header, pack_checkstring, \
+from allmydata.mutable.servermap import ServerMap
+from allmydata.mutable.layout import pack_prefix, pack_share, unpack_header, pack_checkstring, \
      unpack_checkstring, SIGNED_PREFIX
 
 class PublishStatus:
@@ -90,24 +90,16 @@ class Publish:
     To make the initial publish, set servermap to None.
     """
 
-    # we limit the segment size as usual to constrain our memory footprint.
-    # The max segsize is higher for mutable files, because we want to support
-    # dirnodes with up to 10k children, and each child uses about 330 bytes.
-    # If you actually put that much into a directory you'll be using a
-    # footprint of around 14MB, which is higher than we'd like, but it is
-    # more important right now to support large directories than to make
-    # memory usage small when you use them. Once we implement MDMF (with
-    # multiple segments), we will drop this back down, probably to 128KiB.
-    MAX_SEGMENT_SIZE = 3500000
-
-    def __init__(self, filenode, servermap):
+    def __init__(self, filenode, storage_broker, servermap):
         self._node = filenode
+        self._storage_broker = storage_broker
         self._servermap = servermap
         self._storage_index = self._node.get_storage_index()
         self._log_prefix = prefix = si_b2a(self._storage_index)[:5]
-        num = self._node._client.log("Publish(%s): starting" % prefix)
+        num = self.log("Publish(%s): starting" % prefix, parent=None)
         self._log_number = num
         self._running = True
+        self._first_write_error = None
 
         self._status = PublishStatus()
         self._status.set_storage_index(self._storage_index)
@@ -142,10 +134,6 @@ class Publish:
         # 5: when enough responses are back, we're done
 
         self.log("starting publish, datalen is %s" % len(newdata))
-        if len(newdata) > self.MAX_SEGMENT_SIZE:
-            raise FileTooLargeError("SDMF is limited to one segment, and "
-                                    "%d > %d" % (len(newdata),
-                                                 self.MAX_SEGMENT_SIZE))
         self._status.set_size(len(newdata))
         self._status.set_status("Started")
         self._started = time.time()
@@ -189,9 +177,8 @@ class Publish:
         assert self._privkey
         self._encprivkey = self._node.get_encprivkey()
 
-        client = self._node._client
-        full_peerlist = client.get_permuted_peers("storage",
-                                                  self._storage_index)
+        sb = self._storage_broker
+        full_peerlist = sb.get_servers_for_index(self._storage_index)
         self.full_peerlist = full_peerlist # for use later, immutable
         self.bad_peers = set() # peerids who have errbacked/refused requests
 
@@ -262,7 +249,7 @@ class Publish:
         return self.done_deferred
 
     def setup_encoding_parameters(self):
-        segment_size = min(self.MAX_SEGMENT_SIZE, len(self.newdata))
+        segment_size = len(self.newdata)
         # this must be a multiple of self.required_shares
         segment_size = mathutil.next_multiple(segment_size,
                                               self.required_shares)
@@ -382,9 +369,11 @@ class Publish:
         peerlist.sort()
 
         if not peerlist:
-            raise NotEnoughServersError("Ran out of non-bad servers")
+            raise NotEnoughServersError("Ran out of non-bad servers, "
+                                        "first_error=%s" %
+                                        str(self._first_write_error),
+                                        self._first_write_error)
 
-        new_assignments = []
         # we then index this peerlist with an integer, because we may have to
         # wrap. We update the goal as we go.
         i = 0
@@ -475,7 +464,7 @@ class Publish:
             # build the block hash tree. SDMF has only one leaf.
             leaves = [hashutil.block_hash(share_data)]
             t = hashtree.HashTree(leaves)
-            block_hash_trees[shnum] = block_hash_tree = list(t)
+            block_hash_trees[shnum] = list(t)
             share_hash_leaves[shnum] = t[0]
         for leaf in share_hash_leaves:
             assert leaf is not None
@@ -646,6 +635,8 @@ class Publish:
             d.addCallbacks(self._got_write_answer, self._got_write_error,
                            callbackArgs=(peerid, shnums, started),
                            errbackArgs=(peerid, shnums, started))
+            # tolerate immediate errback, like with DeadReferenceError
+            d.addBoth(fireEventually)
             d.addCallback(self.loop)
             d.addErrback(self._fatal_error)
 
@@ -801,6 +792,8 @@ class Publish:
         for shnum in shnums:
             self.outstanding.discard( (peerid, shnum) )
         self.bad_peers.add(peerid)
+        if self._first_write_error is None:
+            self._first_write_error = f
         self.log(format="error while writing shares %(shnums)s to peerid %(peerid)s",
                  shnums=list(shnums), peerid=idlib.shortnodeid_b2a(peerid),
                  failure=f,
@@ -828,7 +821,7 @@ class Publish:
             # TODO: recovery
         else:
             self.log("Publish done, success")
-            self._status.set_status("Done")
+            self._status.set_status("Finished")
             self._status.set_progress(1.0)
         eventually(self.done_deferred.callback, res)
 
