@@ -4,8 +4,7 @@ from itertools import count
 from zope.interface import implements
 from twisted.internet import defer
 from twisted.python import failure
-from foolscap import DeadReferenceError
-from foolscap.eventual import eventually, fireEventually
+from foolscap.api import DeadReferenceError, eventually, fireEventually
 from allmydata.interfaces import IRetrieveStatus, NotEnoughSharesError
 from allmydata.util import hashutil, idlib, log
 from allmydata import hashtree, codec
@@ -13,8 +12,8 @@ from allmydata.storage.server import si_b2a
 from pycryptopp.cipher.aes import AES
 from pycryptopp.publickey import rsa
 
-from common import DictOfSets, CorruptShareError, UncoordinatedWriteError
-from layout import SIGNED_PREFIX, unpack_share_data
+from allmydata.mutable.common import DictOfSets, CorruptShareError, UncoordinatedWriteError
+from allmydata.mutable.layout import SIGNED_PREFIX, unpack_share_data
 
 class RetrieveStatus:
     implements(IRetrieveStatus)
@@ -84,9 +83,9 @@ class Retrieve:
 
     def __init__(self, filenode, servermap, verinfo, fetch_privkey=False):
         self._node = filenode
-        assert self._node._pubkey
+        assert self._node.get_pubkey()
         self._storage_index = filenode.get_storage_index()
-        assert self._node._readkey
+        assert self._node.get_readkey()
         self._last_failure = None
         prefix = si_b2a(self._storage_index)[:5]
         self._log_number = log.msg("Retrieve(%s): starting" % prefix)
@@ -96,13 +95,13 @@ class Retrieve:
         self._bad_shares = set()
 
         self.servermap = servermap
-        assert self._node._pubkey
+        assert self._node.get_pubkey()
         self.verinfo = verinfo
         # during repair, we may be called upon to grab the private key, since
         # it wasn't picked up during a verify=False checker run, and we'll
         # need it for repair to generate the a new version.
         self._need_privkey = fetch_privkey
-        if self._node._privkey:
+        if self._node.get_privkey():
             self._need_privkey = False
 
         self._status = RetrieveStatus()
@@ -199,8 +198,8 @@ class Retrieve:
         got_from_cache = False
         datavs = []
         for (offset, length) in readv:
-            (data, timestamp) = self._node._cache.read(self.verinfo, shnum,
-                                                       offset, length)
+            (data, timestamp) = self._node._read_from_cache(self.verinfo, shnum,
+                                                            offset, length)
             if data is not None:
                 datavs.append(data)
         if len(datavs) == len(readv):
@@ -396,7 +395,7 @@ class Retrieve:
         self._status.timings["fetch"] = elapsed
 
         d = defer.maybeDeferred(self._decode)
-        d.addCallback(self._decrypt, IV, self._node._readkey)
+        d.addCallback(self._decrypt, IV, self._node.get_readkey())
         d.addBoth(self._done)
         return d # purely for test convenience
 
@@ -466,8 +465,7 @@ class Retrieve:
             self.log(format=format,
                      level=log.WEIRD, umid="ezTfjw", **args)
             err = NotEnoughSharesError("%s, last failure: %s" %
-                                      (format % args, self._last_failure),
-                                       len(self.shares), k)
+                                      (format % args, self._last_failure))
             if self._bad_shares:
                 self.log("We found some bad shares this pass. You should "
                          "update the servermap and try again to check "
@@ -539,7 +537,7 @@ class Retrieve:
             self._status.set_status("Failed")
         else:
             self.log("Retrieve done, success!")
-            self._status.set_status("Done")
+            self._status.set_status("Finished")
             self._status.set_progress(1.0)
             # remember the encoding parameters, use them again next time
             (seqnum, root_hash, IV, segsize, datalength, k, N, prefix,
