@@ -1,33 +1,38 @@
 
 import os, sys
-from twisted.python import usage
-from allmydata.scripts.common import BasedirMixin, NoDefaultBasedirMixin
+from allmydata.scripts.common import BasedirMixin, BaseOptions
+from allmydata.util.assertutil import precondition
+from allmydata.util.encodingutil import listdir_unicode, argv_to_unicode, quote_output
+import allmydata
 
-class CreateClientOptions(BasedirMixin, usage.Options):
+class CreateClientOptions(BasedirMixin, BaseOptions):
     optParameters = [
-        ("basedir", "C", None, "which directory to create the client in"),
-        # we provide create-client -time options for the most common
+        # we provide 'create-node'-time options for the most common
         # configuration knobs. The rest can be controlled by editing
         # tahoe.cfg before node startup.
-        ("nickname", "n", None, "nickname for this node"),
-        ("introducer", "i", None, "introducer FURL to use"),
+        ("nickname", "n", None, "Specify the nickname for this node."),
+        ("introducer", "i", None, "Specify the introducer FURL to use."),
         ("webport", "p", "tcp:3456:interface=127.0.0.1",
-         "which TCP port to run the HTTP interface on. Use 'none' to disable."),
-        ]
-    optFlags = [
-        ("no-storage", None, "do not offer storage service to other nodes"),
+         "Specify which TCP port to run the HTTP interface on. Use 'none' to disable."),
         ]
 
-class CreateIntroducerOptions(NoDefaultBasedirMixin, usage.Options):
-    optParameters = [
-        ["basedir", "C", None, "which directory to create the introducer in"],
+class CreateNodeOptions(CreateClientOptions):
+    optFlags = [
+        ("no-storage", None, "Do not offer storage service to other nodes."),
         ]
 
+class CreateIntroducerOptions(BasedirMixin, BaseOptions):
+    default_nodedir = None
+
+    optParameters = [
+        ["node-directory", "d", None, "Specify which directory the introducer should be created in. [no default]"],
+    ]
+
 client_tac = """
 # -*- python -*-
 
 import pkg_resources
-pkg_resources.require('allmydata-tahoe')
+pkg_resources.require('%s')
 pkg_resources.require('twisted')
 from allmydata import client
 from twisted.application import service
@@ -36,13 +41,13 @@ c = client.Client()
 
 application = service.Application("allmydata_client")
 c.setServiceParent(application)
-"""
+""" % (allmydata.__appname__,)
 
 introducer_tac = """
 # -*- python -*-
 
 import pkg_resources
-pkg_resources.require('allmydata-tahoe')
+pkg_resources.require('%s')
 pkg_resources.require('twisted')
 from allmydata import introducer
 from twisted.application import service
@@ -51,7 +56,7 @@ c = introducer.IntroducerNode()
 
 application = service.Application("allmydata_introducer")
 c.setServiceParent(application)
-"""
+""" % (allmydata.__appname__,)
 
 def write_node_config(c, config):
     # this is shared between clients and introducers
@@ -65,11 +70,14 @@ def write_node_config(c, config):
     c.write("\n\n")
 
     c.write("[node]\n")
-    c.write("nickname = %s\n" % config.get("nickname", "")) #TODO: utf8 in argv?
-    webport = config.get("webport", "none")
+    nickname = argv_to_unicode(config.get("nickname") or "")
+    c.write("nickname = %s\n" % (nickname.encode('utf-8'),))
+
+    # TODO: validate webport
+    webport = argv_to_unicode(config.get("webport") or "none")
     if webport.lower() == "none":
         webport = ""
-    c.write("web.port = %s\n" % webport)
+    c.write("web.port = %s\n" % (webport.encode('utf-8'),))
     c.write("web.static = public_html\n")
     c.write("#tub.port =\n")
     c.write("#tub.location = \n")
@@ -81,10 +89,13 @@ def write_node_config(c, config):
     c.write("\n")
 
 
-def create_client(basedir, config, out=sys.stdout, err=sys.stderr):
+def create_node(basedir, config, out=sys.stdout, err=sys.stderr):
+    # This should always be called with an absolute Unicode basedir.
+    precondition(isinstance(basedir, unicode), basedir)
+
     if os.path.exists(basedir):
-        if os.listdir(basedir):
-            print >>err, "The base directory \"%s\", which is \"%s\" is not empty." % (basedir, os.path.abspath(basedir))
+        if listdir_unicode(basedir):
+            print >>err, "The base directory %s is not empty." % quote_output(basedir)
             print >>err, "To avoid clobbering anything, I am going to quit now."
             print >>err, "Please use a different directory, or empty this one."
             return -1
@@ -115,6 +126,8 @@ def create_client(basedir, config, out=sys.stdout, err=sys.stderr):
     c.write("enabled = %s\n" % boolstr[storage_enabled])
     c.write("#readonly =\n")
     c.write("#reserved_space =\n")
+    c.write("#expire.enabled =\n")
+    c.write("#expire.mode =\n")
     c.write("\n")
 
     c.write("[helper]\n")
@@ -125,17 +138,26 @@ def create_client(basedir, config, out=sys.stdout, err=sys.stderr):
 
     from allmydata.util import fileutil
     fileutil.make_dirs(os.path.join(basedir, "private"), 0700)
-    print >>out, "client created in %s" % basedir
+    print >>out, "Node created in %s" % quote_output(basedir)
     if not config.get("introducer", ""):
         print >>out, " Please set [client]introducer.furl= in tahoe.cfg!"
         print >>out, " The node cannot connect to a grid without it."
     if not config.get("nickname", ""):
         print >>out, " Please set [node]nickname= in tahoe.cfg"
 
+
+def create_client(basedir, config, out=sys.stdout, err=sys.stderr):
+    config['no-storage'] = True
+    return create_node(basedir, config, out=out, err=err)
+
+
 def create_introducer(basedir, config, out=sys.stdout, err=sys.stderr):
+    # This should always be called with an absolute Unicode basedir.
+    precondition(isinstance(basedir, unicode), basedir)
+
     if os.path.exists(basedir):
-        if os.listdir(basedir):
-            print >>err, "The base directory \"%s\", which is \"%s\" is not empty." % (basedir, os.path.abspath(basedir))
+        if listdir_unicode(basedir):
+            print >>err, "The base directory %s is not empty." % quote_output(basedir)
             print >>err, "To avoid clobbering anything, I am going to quit now."
             print >>err, "Please use a different directory, or empty this one."
             return -1
@@ -150,15 +172,17 @@ def create_introducer(basedir, config, out=sys.stdout, err=sys.stderr):
     write_node_config(c, config)
     c.close()
 
-    print >>out, "introducer created in %s" % basedir
+    print >>out, "Introducer created in %s" % quote_output(basedir)
+
 
 subCommands = [
-    ["create-client", None, CreateClientOptions, "Create a client node."],
-    ["create-introducer", None, CreateIntroducerOptions, "Create a introducer node."],
-
+    ["create-node", None, CreateNodeOptions, "Create a node that acts as a client, server or both."],
+    ["create-client", None, CreateClientOptions, "Create a client node (with storage initially disabled)."],
+    ["create-introducer", None, CreateIntroducerOptions, "Create an introducer node."],
 ]
 
 dispatch = {
+    "create-node": create_node,
     "create-client": create_client,
     "create-introducer": create_introducer,
     }
