@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 from twisted.trial import unittest
 
 from twisted.python import usage, runtime
@@ -5,39 +7,163 @@ from twisted.internet import utils
 import os.path, re, sys
 from cStringIO import StringIO
 from allmydata.util import fileutil, pollmixin
+from allmydata.util.encodingutil import unicode_to_argv, unicode_to_output, get_filesystem_encoding
 from allmydata.scripts import runner
 
 from allmydata.test import common_util
 import allmydata
 
-bintahoe = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(allmydata.__file__))), 'bin', 'tahoe')
-if sys.platform == "win32":
-    bintahoe += ".exe"
+timeout = 240
 
-class SkipOnCygwinMixin:
-    def skip_on_cygwin(self):
+srcfile = allmydata.__file__
+srcdir = os.path.dirname(os.path.dirname(os.path.normcase(os.path.realpath(srcfile))))
+
+rootdir = os.path.dirname(srcdir)
+if os.path.basename(srcdir) == 'site-packages':
+    if re.search(r'python.+\..+', os.path.basename(rootdir)):
+        rootdir = os.path.dirname(rootdir)
+    rootdir = os.path.dirname(rootdir)
+
+bintahoe = os.path.join(rootdir, 'bin', 'tahoe')
+if sys.platform == "win32":
+    bintahoe += ".pyscript"
+    if not os.path.exists(bintahoe):
+       alt_bintahoe = os.path.join(rootdir, 'Scripts', 'tahoe.pyscript')
+       if os.path.exists(alt_bintahoe):
+           bintahoe = alt_bintahoe
+
+
+class SkipMixin:
+    def skip_if_cannot_run_bintahoe(self):
         if "cygwin" in sys.platform.lower():
             raise unittest.SkipTest("We don't know how to make this test work on cygwin: spawnProcess seems to hang forever. We don't know if 'bin/tahoe start' can be run on cygwin.")
-
-class TheRightCode(common_util.SignalMixin, unittest.TestCase,
-                   SkipOnCygwinMixin):
-    def test_path(self):
-        self.skip_on_cygwin()
         if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
+            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location (%s), and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of Tahoe. Perhaps running 'setup.py build' again will help." % (bintahoe,))
+
+    def skip_if_cannot_daemonize(self):
+        self.skip_if_cannot_run_bintahoe()
+        if runtime.platformType == "win32":
+            # twistd on windows doesn't daemonize. cygwin should work normally.
+            raise unittest.SkipTest("twistd does not fork under windows")
+
+
+class BinTahoe(common_util.SignalMixin, unittest.TestCase, SkipMixin):
+    def test_the_right_code(self):
+        cwd = os.path.normcase(os.path.realpath("."))
+        root_from_cwd = os.path.dirname(cwd)
+
+        same = (root_from_cwd == rootdir)
+        if not same:
+            try:
+                same = os.path.samefile(root_from_cwd, rootdir)
+            except AttributeError, e:
+                e  # hush pyflakes
+
+        if not same:
+            msg = ("We seem to be testing the code at %r,\n"
+                   "(according to the source filename %r),\n"
+                   "but expected to be testing the code at %r.\n"
+                   % (rootdir, srcfile, root_from_cwd))
+            if (not isinstance(cwd, unicode) and
+                cwd.decode(get_filesystem_encoding(), 'replace') != os.path.normcase(os.path.realpath(os.getcwdu()))):
+                msg += ("However, this may be a false alarm because the current directory path\n"
+                        "is not representable in the filesystem encoding. Please run the tests\n"
+                        "from the root of the Tahoe-LAFS distribution at a non-Unicode path.")
+                raise unittest.SkipTest(msg)
+            else:
+                msg += "Please run the tests from the root of the Tahoe-LAFS distribution."
+                self.fail(msg)
+
+    def test_path(self):
+        self.skip_if_cannot_run_bintahoe()
         d = utils.getProcessOutputAndValue(bintahoe, args=["--version-and-path"], env=os.environ)
         def _cb(res):
             out, err, rc_or_sig = res
-            self.failUnlessEqual(rc_or_sig, 0)
+            self.failUnlessEqual(rc_or_sig, 0, str(res))
 
-            # Fail unless the allmydata-tahoe package is *this* version *and* was loaded from *this* source directory.
-            required_ver_and_path = "allmydata-tahoe: %s (%s)" % (allmydata.__version__, os.path.dirname(os.path.dirname(allmydata.__file__)))
-            self.failUnless(out.startswith(required_ver_and_path), (out, err, rc_or_sig, required_ver_and_path))
+            # Fail unless the allmydata-tahoe package is *this* version *and*
+            # was loaded from *this* source directory.
+
+            verstr = str(allmydata.__version__)
+
+            self.failIfEqual(verstr, "unknown",
+                             "We don't know our version, because this distribution didn't come "
+                             "with a _version.py and 'setup.py darcsver' hasn't been run.")
+
+            # The Python "rational version numbering" convention
+            # disallows "-r$REV" but allows ".post$REV"
+            # instead. Eventually we'll probably move to that. When we
+            # do, this test won't go red:
+
+            ix = verstr.rfind('-r')
+            if ix != -1:
+                altverstr = verstr[:ix] + '.post' + verstr[ix+2:]
+            else:
+                ix = verstr.rfind('.post')
+                if ix != -1:
+                    altverstr = verstr[:ix] + '-r' + verstr[ix+5:]
+                else:
+                    altverstr = verstr
+
+            required_ver_and_path = "%s: %s (%s)" % (allmydata.__appname__, verstr, srcdir)
+            alt_required_ver_and_path = "%s: %s (%s)" % (allmydata.__appname__, altverstr, srcdir)
+
+            self.failUnless(out.startswith(required_ver_and_path) or out.startswith(alt_required_ver_and_path), (out, err, rc_or_sig, required_ver_and_path))
         d.addCallback(_cb)
         return d
 
+    def test_unicode_arguments_and_output(self):
+        self.skip_if_cannot_run_bintahoe()
+
+        tricky = u"\u2621"
+        try:
+            tricky_arg = unicode_to_argv(tricky, mangle=True)
+            tricky_out = unicode_to_output(tricky)
+        except UnicodeEncodeError:
+            raise unittest.SkipTest("A non-ASCII argument/output could not be encoded on this platform.")
+
+        d = utils.getProcessOutputAndValue(bintahoe, args=[tricky_arg], env=os.environ)
+        def _cb(res):
+            out, err, rc_or_sig = res
+            self.failUnlessEqual(rc_or_sig, 1, str(res))
+            self.failUnlessIn("Unknown command: "+tricky_out, out)
+        d.addCallback(_cb)
+        return d
+
+    def test_run_with_python_options(self):
+        self.skip_if_cannot_run_bintahoe()
+
+        # -t is a harmless option that warns about tabs.
+        d = utils.getProcessOutputAndValue(sys.executable, args=['-t', bintahoe, '--version'],
+                                           env=os.environ)
+        def _cb(res):
+            out, err, rc_or_sig = res
+            self.failUnlessEqual(rc_or_sig, 0, str(res))
+            self.failUnless(out.startswith(allmydata.__appname__+':'), str(res))
+        d.addCallback(_cb)
+        return d
+
+    def test_version_no_noise(self):
+        self.skip_if_cannot_run_bintahoe()
+        import pkg_resources
+        try:
+            pkg_resources.require("Twisted>=9.0.0")
+        except pkg_resources.VersionConflict:
+            raise unittest.SkipTest("We pass this test only with Twisted >= v9.0.0")
+
+        d = utils.getProcessOutputAndValue(bintahoe, args=["--version"], env=os.environ)
+        def _cb(res):
+            out, err, rc_or_sig = res
+            self.failUnlessEqual(rc_or_sig, 0, str(res))
+            self.failUnless(out.startswith(allmydata.__appname__+':'), str(res))
+            self.failIfIn("DeprecationWarning", out, str(res))
+            self.failUnlessEqual(err, "", str(res))
+        d.addCallback(_cb)
+        return d
+
+
 class CreateNode(unittest.TestCase):
-    # exercise "tahoe create-client", create-introducer,
+    # exercise "tahoe create-node", create-introducer,
     # create-key-generator, and create-stats-gatherer, by calling the
     # corresponding code as a subroutine.
 
@@ -51,18 +177,33 @@ class CreateNode(unittest.TestCase):
         rc = runner.runner(argv, stdout=out, stderr=err)
         return rc, out.getvalue(), err.getvalue()
 
-    def test_client(self):
-        basedir = self.workdir("test_client")
-        c1 = os.path.join(basedir, "c1")
-        argv = ["--quiet", "create-client", "--basedir", c1]
+    def do_create(self, kind):
+        basedir = self.workdir("test_" + kind)
+        command = "create-" + kind
+        is_client = kind in ("node", "client")
+        tac = is_client and "tahoe-client.tac" or ("tahoe-" + kind + ".tac")
+
+        n1 = os.path.join(basedir, command + "-n1")
+        argv = ["--quiet", command, "--basedir", n1]
         rc, out, err = self.run_tahoe(argv)
         self.failUnlessEqual(err, "")
         self.failUnlessEqual(out, "")
         self.failUnlessEqual(rc, 0)
-        self.failUnless(os.path.exists(c1))
-        self.failUnless(os.path.exists(os.path.join(c1, "tahoe-client.tac")))
+        self.failUnless(os.path.exists(n1))
+        self.failUnless(os.path.exists(os.path.join(n1, tac)))
 
-        # creating the client a second time should be rejected
+        if is_client:
+            # tahoe.cfg should exist, and should have storage enabled for
+            # 'create-node', and disabled for 'create-client'.
+            tahoe_cfg = os.path.join(n1, "tahoe.cfg")
+            self.failUnless(os.path.exists(tahoe_cfg))
+            content = open(tahoe_cfg).read()
+            if kind == "client":
+                self.failUnless("\n[storage]\nenabled = false\n" in content)
+            else:
+                self.failUnless("\n[storage]\nenabled = true\n" in content)
+
+        # creating the node a second time should be rejected
         rc, out, err = self.run_tahoe(argv)
         self.failIfEqual(rc, 0, str((out, err, rc)))
         self.failUnlessEqual(out, "")
@@ -74,130 +215,67 @@ class CreateNode(unittest.TestCase):
             self.failIf(re.search("[\S][^\.!?]$", line), (line,))
 
         # test that the non --basedir form works too
-        c2 = os.path.join(basedir, "c2")
-        argv = ["--quiet", "create-client", c2]
+        n2 = os.path.join(basedir, command + "-n2")
+        argv = ["--quiet", command, n2]
         rc, out, err = self.run_tahoe(argv)
-        self.failUnless(os.path.exists(c2))
-        self.failUnless(os.path.exists(os.path.join(c2, "tahoe-client.tac")))
+        self.failUnlessEqual(err, "")
+        self.failUnlessEqual(out, "")
+        self.failUnlessEqual(rc, 0)
+        self.failUnless(os.path.exists(n2))
+        self.failUnless(os.path.exists(os.path.join(n2, tac)))
 
-        # make sure it rejects too many arguments
-        argv = ["create-client", "basedir", "extraarg"]
+        # test the --node-directory form
+        n3 = os.path.join(basedir, command + "-n3")
+        argv = ["--quiet", command, "--node-directory", n3]
+        rc, out, err = self.run_tahoe(argv)
+        self.failUnlessEqual(err, "")
+        self.failUnlessEqual(out, "")
+        self.failUnlessEqual(rc, 0)
+        self.failUnless(os.path.exists(n3))
+        self.failUnless(os.path.exists(os.path.join(n3, tac)))
+
+        # test the --multiple form
+        n4 = os.path.join(basedir, command + "-n4")
+        n5 = os.path.join(basedir, command + "-n5")
+        argv = ["--quiet", command, "--multiple", n4, n5]
+        rc, out, err = self.run_tahoe(argv)
+        self.failUnlessEqual(err, "")
+        self.failUnlessEqual(out, "")
+        self.failUnlessEqual(rc, 0)
+        self.failUnless(os.path.exists(n4))
+        self.failUnless(os.path.exists(os.path.join(n4, tac)))
+        self.failUnless(os.path.exists(n5))
+        self.failUnless(os.path.exists(os.path.join(n5, tac)))
+
+        # make sure it rejects too many arguments without --multiple
+        argv = [command, "basedir", "extraarg"]
         self.failUnlessRaises(usage.UsageError,
                               runner.runner, argv,
                               run_by_human=False)
 
+        # when creating a non-client, there is no default for the basedir
+        if not is_client:
+            argv = [command]
+            self.failUnlessRaises(usage.UsageError,
+                                  runner.runner, argv,
+                                  run_by_human=False)
+
+
+    def test_node(self):
+        self.do_create("node")
+
+    def test_client(self):
+        # create-client should behave like create-node --no-storage.
+        self.do_create("client")
+
     def test_introducer(self):
-        basedir = self.workdir("test_introducer")
-        c1 = os.path.join(basedir, "c1")
-        argv = ["--quiet", "create-introducer", "--basedir", c1]
-        rc, out, err = self.run_tahoe(argv)
-        self.failUnlessEqual(err, "", err)
-        self.failUnlessEqual(out, "")
-        self.failUnlessEqual(rc, 0)
-        self.failUnless(os.path.exists(c1))
-        self.failUnless(os.path.exists(os.path.join(c1,"tahoe-introducer.tac")))
-
-        # creating the introducer a second time should be rejected
-        rc, out, err = self.run_tahoe(argv)
-        self.failIfEqual(rc, 0)
-        self.failUnlessEqual(out, "")
-        self.failUnless("is not empty" in err)
-
-        # Fail if there is a non-empty line that doesn't end with a
-        # punctuation mark.
-        for line in err.splitlines():
-            self.failIf(re.search("[\S][^\.!?]$", line), (line,))
-
-        # test the non --basedir form
-        c2 = os.path.join(basedir, "c2")
-        argv = ["--quiet", "create-introducer", c2]
-        rc, out, err = self.run_tahoe(argv)
-        self.failUnlessEqual(err, "", err)
-        self.failUnlessEqual(out, "")
-        self.failUnlessEqual(rc, 0)
-        self.failUnless(os.path.exists(c2))
-        self.failUnless(os.path.exists(os.path.join(c2,"tahoe-introducer.tac")))
-
-        # reject extra arguments
-        argv = ["create-introducer", "basedir", "extraarg"]
-        self.failUnlessRaises(usage.UsageError,
-                              runner.runner, argv,
-                              run_by_human=False)
-        # and require basedir to be provided in some form
-        argv = ["create-introducer"]
-        self.failUnlessRaises(usage.UsageError,
-                              runner.runner, argv,
-                              run_by_human=False)
+        self.do_create("introducer")
 
     def test_key_generator(self):
-        basedir = self.workdir("test_key_generator")
-        kg1 = os.path.join(basedir, "kg1")
-        argv = ["--quiet", "create-key-generator", "--basedir", kg1]
-        rc, out, err = self.run_tahoe(argv)
-        self.failUnlessEqual(err, "")
-        self.failUnlessEqual(out, "")
-        self.failUnlessEqual(rc, 0)
-        self.failUnless(os.path.exists(kg1))
-        self.failUnless(os.path.exists(os.path.join(kg1, "tahoe-key-generator.tac")))
-
-        # creating it a second time should be rejected
-        rc, out, err = self.run_tahoe(argv)
-        self.failIfEqual(rc, 0, str((out, err, rc)))
-        self.failUnlessEqual(out, "")
-        self.failUnless("is not empty." in err)
-
-        # make sure it rejects too many arguments
-        argv = ["create-key-generator", "basedir", "extraarg"]
-        self.failUnlessRaises(usage.UsageError,
-                              runner.runner, argv,
-                              run_by_human=False)
-
-        # make sure it rejects a missing basedir specification
-        argv = ["create-key-generator"]
-        rc, out, err = self.run_tahoe(argv)
-        self.failIfEqual(rc, 0, str((out, err, rc)))
-        self.failUnlessEqual(out, "")
-        self.failUnless("a basedir was not provided" in err)
+        self.do_create("key-generator")
 
     def test_stats_gatherer(self):
-        basedir = self.workdir("test_stats_gatherer")
-        sg1 = os.path.join(basedir, "sg1")
-        argv = ["--quiet", "create-stats-gatherer", "--basedir", sg1]
-        rc, out, err = self.run_tahoe(argv)
-        self.failUnlessEqual(err, "")
-        self.failUnlessEqual(out, "")
-        self.failUnlessEqual(rc, 0)
-        self.failUnless(os.path.exists(sg1))
-        self.failUnless(os.path.exists(os.path.join(sg1, "tahoe-stats-gatherer.tac")))
-
-        # creating it a second time should be rejected
-        rc, out, err = self.run_tahoe(argv)
-        self.failIfEqual(rc, 0, str((out, err, rc)))
-        self.failUnlessEqual(out, "")
-        self.failUnless("is not empty." in err)
-
-        # test the non --basedir form
-        kg2 = os.path.join(basedir, "kg2")
-        argv = ["--quiet", "create-stats-gatherer", kg2]
-        rc, out, err = self.run_tahoe(argv)
-        self.failUnlessEqual(err, "", err)
-        self.failUnlessEqual(out, "")
-        self.failUnlessEqual(rc, 0)
-        self.failUnless(os.path.exists(kg2))
-        self.failUnless(os.path.exists(os.path.join(kg2,"tahoe-stats-gatherer.tac")))
-
-        # make sure it rejects too many arguments
-        argv = ["create-stats-gatherer", "basedir", "extraarg"]
-        self.failUnlessRaises(usage.UsageError,
-                              runner.runner, argv,
-                              run_by_human=False)
-
-        # make sure it rejects a missing basedir specification
-        argv = ["create-stats-gatherer"]
-        rc, out, err = self.run_tahoe(argv)
-        self.failIfEqual(rc, 0, str((out, err, rc)))
-        self.failUnlessEqual(out, "")
-        self.failUnless("a basedir was not provided" in err)
+        self.do_create("stats-gatherer")
 
     def test_subcommands(self):
         # no arguments should trigger a command listing, via UsageError
@@ -208,7 +286,7 @@ class CreateNode(unittest.TestCase):
 
 
 class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
-              SkipOnCygwinMixin):
+              SkipMixin):
     # exercise "tahoe start", for both introducer, client node, and
     # key-generator, by spawning "tahoe start" as a subprocess. This doesn't
     # get us figleaf-based line-level coverage, but it does a better job of
@@ -227,12 +305,7 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
         return basedir
 
     def test_introducer(self):
-        self.skip_on_cygwin()
-        if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
-        if runtime.platformType == "win32":
-            # twistd on windows doesn't daemonize. cygwin works normally.
-            raise unittest.SkipTest("twistd does not fork under windows")
+        self.skip_if_cannot_daemonize()
         basedir = self.workdir("test_introducer")
         c1 = os.path.join(basedir, "c1")
         HOTLINE_FILE = os.path.join(c1, "suicide_prevention_hotline")
@@ -327,11 +400,15 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
             return res
         d.addBoth(_remove_hotline)
         return d
+    test_introducer.timeout = 480 # This hit the 120-second timeout on "François Lenny-armv5tel", then it hit a 240-second timeout on our feisty2.5 buildslave: http://allmydata.org/buildbot/builders/feisty2.5/builds/2381/steps/test/logs/test.log
 
     def test_client_no_noise(self):
-        self.skip_on_cygwin()
-        if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
+        self.skip_if_cannot_daemonize()
+        import pkg_resources
+        try:
+            pkg_resources.require("Twisted>=9.0.0")
+        except pkg_resources.VersionConflict:
+            raise unittest.SkipTest("We pass this test only with Twisted >= v9.0.0")
         basedir = self.workdir("test_client_no_noise")
         c1 = os.path.join(basedir, "c1")
         HOTLINE_FILE = os.path.join(c1, "suicide_prevention_hotline")
@@ -386,22 +463,16 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
             return utils.getProcessOutputAndValue(bintahoe, args=["--quiet", "stop", c1], env=os.environ)
         d.addCallback(_stop)
         return d
-    test_client_no_noise.todo = "We submitted a patch to Nevow to silence this warning: http://divmod.org/trac/ticket/2830"
 
     def test_client(self):
-        self.skip_on_cygwin()
-        if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
-        if runtime.platformType == "win32":
-            # twistd on windows doesn't daemonize. cygwin works normally.
-            raise unittest.SkipTest("twistd does not fork under windows")
+        self.skip_if_cannot_daemonize()
         basedir = self.workdir("test_client")
         c1 = os.path.join(basedir, "c1")
         HOTLINE_FILE = os.path.join(c1, "suicide_prevention_hotline")
         TWISTD_PID_FILE = os.path.join(c1, "twistd.pid")
         PORTNUMFILE = os.path.join(c1, "client.port")
 
-        d = utils.getProcessOutputAndValue(bintahoe, args=["--quiet", "create-client", "--basedir", c1, "--webport", "0"], env=os.environ)
+        d = utils.getProcessOutputAndValue(bintahoe, args=["--quiet", "create-node", "--basedir", c1, "--webport", "0"], env=os.environ)
         def _cb(res):
             out, err, rc_or_sig = res
             self.failUnlessEqual(rc_or_sig, 0)
@@ -420,7 +491,7 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
         def _cb2(res):
             out, err, rc_or_sig = res
             open(HOTLINE_FILE, "w").write("")
-            errstr = "cc=%d, OUT: '%s', ERR: '%s'" % (rc_or_sig, out, err)
+            errstr = "rc=%d, OUT: '%s', ERR: '%s'" % (rc_or_sig, out, err)
             self.failUnlessEqual(rc_or_sig, 0, errstr)
             self.failUnlessEqual(out, "", errstr)
             # self.failUnlessEqual(err, "", errstr) # See test_client_no_noise -- for now we ignore noise.
@@ -492,9 +563,7 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
         return d
 
     def test_baddir(self):
-        self.skip_on_cygwin()
-        if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
+        self.skip_if_cannot_daemonize()
         basedir = self.workdir("test_baddir")
         fileutil.make_dirs(basedir)
 
@@ -502,7 +571,7 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
         def _cb(res):
             out, err, rc_or_sig = res
             self.failUnlessEqual(rc_or_sig, 1)
-            self.failUnless("does not look like a node directory" in err)
+            self.failUnless("does not look like a node directory" in err, err)
         d.addCallback(_cb)
 
         def _then_stop_it(res):
@@ -528,12 +597,7 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
         return d
 
     def test_keygen(self):
-        self.skip_on_cygwin()
-        if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
-        if runtime.platformType == "win32":
-            # twistd on windows doesn't daemonize. cygwin works normally.
-            raise unittest.SkipTest("twistd does not fork under windows")
+        self.skip_if_cannot_daemonize()
         basedir = self.workdir("test_keygen")
         c1 = os.path.join(basedir, "c1")
         TWISTD_PID_FILE = os.path.join(c1, "twistd.pid")
