@@ -1,13 +1,18 @@
 
+import simplejson
 from twisted.web import http, server
+from twisted.python import log
 from zope.interface import Interface
 from nevow import loaders, appserver
 from nevow.inevow import IRequest
 from nevow.util import resource_filename
 from allmydata.interfaces import ExistingChildError, NoSuchChildError, \
-     FileTooLargeError, NotEnoughSharesError
+     FileTooLargeError, NotEnoughSharesError, NoSharesError, \
+     EmptyPathnameComponentError, MustBeDeepImmutableError, \
+     MustBeReadonlyError, MustNotBeUnknownRWError
 from allmydata.mutable.common import UnrecoverableFileError
-from allmydata.util import abbreviate # TODO: consolidate
+from allmydata.util import abbreviate
+from allmydata.util.encodingutil import to_str
 
 class IOpHandleTable(Interface):
     pass
@@ -20,6 +25,12 @@ def boolean_of_arg(arg):
     assert arg.lower() in ("true", "t", "1", "false", "f", "0", "on", "off")
     return arg.lower() in ("true", "t", "1", "on")
 
+def parse_replace_arg(replace):
+    if replace.lower() == "only-files":
+        return replace
+    else:
+        return boolean_of_arg(replace)
+
 def get_root(ctx_or_req):
     req = IRequest(ctx_or_req)
     # the addSlash=True gives us one extra (empty) segment
@@ -46,6 +57,24 @@ def get_arg(ctx_or_req, argname, default=None, multiple=False):
         return results[0]
     return default
 
+def convert_children_json(nodemaker, children_json):
+    """I convert the JSON output of GET?t=json into the dict-of-nodes input
+    to both dirnode.create_subdirectory() and
+    client.create_directory(initial_children=). This is used by
+    t=mkdir-with-children and t=mkdir-immutable"""
+    children = {}
+    if children_json:
+        data = simplejson.loads(children_json)
+        for (namex, (ctype, propdict)) in data.iteritems():
+            namex = unicode(namex)
+            writecap = to_str(propdict.get("rw_uri"))
+            readcap = to_str(propdict.get("ro_uri"))
+            metadata = propdict.get("metadata", {})
+            # name= argument is just for error reporting
+            childnode = nodemaker.create_from_cap(writecap, readcap, name=namex)
+            children[namex] = (childnode, metadata)
+    return children
+
 def abbreviate_time(data):
     # 1.23s, 790ms, 132us
     if data is None:
@@ -56,10 +85,23 @@ def abbreviate_time(data):
     if s >= 1.0:
         return "%.2fs" % s
     if s >= 0.01:
-        return "%dms" % (1000*s)
+        return "%.0fms" % (1000*s)
     if s >= 0.001:
         return "%.1fms" % (1000*s)
-    return "%dus" % (1000000*s)
+    return "%.0fus" % (1000000*s)
+
+def compute_rate(bytes, seconds):
+    if bytes is None:
+      return None
+
+    if seconds is None or seconds == 0:
+      return None
+
+    # negative values don't make sense here
+    assert bytes > -1
+    assert seconds > 0
+
+    return 1.0 * bytes / seconds
 
 def abbreviate_rate(data):
     # 21.8kBps, 554.4kBps 4.37MBps
@@ -70,7 +112,7 @@ def abbreviate_rate(data):
         return "%1.2fMBps" % (r/1000000)
     if r > 1000:
         return "%.1fkBps" % (r/1000)
-    return "%dBps" % r
+    return "%.0fBps" % r
 
 def abbreviate_size(data):
     # 21.8kB, 554.4kB 4.37MB
@@ -83,7 +125,7 @@ def abbreviate_size(data):
         return "%1.2fMB" % (r/1000000)
     if r > 1000:
         return "%.1fkB" % (r/1000)
-    return "%dB" % r
+    return "%.0fB" % r
 
 def plural(sequence_or_length):
     if isinstance(sequence_or_length, int):
@@ -117,6 +159,9 @@ def should_create_intermediate_directories(req):
 
 def humanize_failure(f):
     # return text, responsecode
+    if f.check(EmptyPathnameComponentError):
+        return ("The webapi does not allow empty pathname components, "
+                "i.e. a double slash", http.BAD_REQUEST)
     if f.check(ExistingChildError):
         return ("There was already a child by that name, and you asked me "
                 "to not replace it.", http.CONFLICT)
@@ -124,21 +169,20 @@ def humanize_failure(f):
         name = f.value.args[0]
         return ("No such child: %s" % name.encode("utf-8"), http.NOT_FOUND)
     if f.check(NotEnoughSharesError):
-        got = f.value.got
-        needed = f.value.needed
-        if got == 0:
-            t = ("NotEnoughSharesError: no shares could be found. "
-                 "Zero shares usually indicates a corrupt URI, or that "
-                 "no servers were connected, but it might also indicate "
-                 "severe corruption. You should perform a filecheck on "
-                 "this object to learn more.")
-        else:
-            t = ("NotEnoughSharesError: %d share%s found, but we need "
-                 "%d to recover the file. This indicates that some "
-                 "servers were unavailable, or that shares have been "
-                 "lost to server departure, hard drive failure, or disk "
-                 "corruption. You should perform a filecheck on "
-                 "this object to learn more.") % (got, plural(got), needed)
+        t = ("NotEnoughSharesError: This indicates that some "
+             "servers were unavailable, or that shares have been "
+             "lost to server departure, hard drive failure, or disk "
+             "corruption. You should perform a filecheck on "
+             "this object to learn more.\n\nThe full error message is:\n"
+             "%s") % str(f.value)
+        return (t, http.GONE)
+    if f.check(NoSharesError):
+        t = ("NoSharesError: no shares could be found. "
+             "Zero shares usually indicates a corrupt URI, or that "
+             "no servers were connected, but it might also indicate "
+             "severe corruption. You should perform a filecheck on "
+             "this object to learn more.\n\nThe full error message is:\n"
+             "%s") % str(f.value)
         return (t, http.GONE)
     if f.check(UnrecoverableFileError):
         t = ("UnrecoverableFileError: the directory (or mutable file) could "
@@ -149,6 +193,43 @@ def humanize_failure(f):
              "failure, or disk corruption. You should perform a filecheck on "
              "this object to learn more.")
         return (t, http.GONE)
+    if f.check(MustNotBeUnknownRWError):
+        name = f.value.args[1]
+        immutable = f.value.args[2]
+        if immutable:
+            t = ("MustNotBeUnknownRWError: an operation to add a child named "
+                 "'%s' to a directory was given an unknown cap in a write slot.\n"
+                 "If the cap is actually an immutable readcap, then using a "
+                 "webapi server that supports a later version of Tahoe may help.\n\n"
+                 "If you are using the webapi directly, then specifying an immutable "
+                 "readcap in the read slot (ro_uri) of the JSON PROPDICT, and "
+                 "omitting the write slot (rw_uri), would also work in this "
+                 "case.") % name.encode("utf-8")
+        else:
+            t = ("MustNotBeUnknownRWError: an operation to add a child named "
+                 "'%s' to a directory was given an unknown cap in a write slot.\n"
+                 "Using a webapi server that supports a later version of Tahoe "
+                 "may help.\n\n"
+                 "If you are using the webapi directly, specifying a readcap in "
+                 "the read slot (ro_uri) of the JSON PROPDICT, as well as a "
+                 "writecap in the write slot if desired, would also work in this "
+                 "case.") % name.encode("utf-8")
+        return (t, http.BAD_REQUEST)
+    if f.check(MustBeDeepImmutableError):
+        name = f.value.args[1]
+        t = ("MustBeDeepImmutableError: a cap passed to this operation for "
+             "the child named '%s', needed to be immutable but was not. Either "
+             "the cap is being added to an immutable directory, or it was "
+             "originally retrieved from an immutable directory as an unknown "
+             "cap." % name.encode("utf-8"))
+        return (t, http.BAD_REQUEST)
+    if f.check(MustBeReadonlyError):
+        name = f.value.args[1]
+        t = ("MustBeReadonlyError: a cap passed to this operation for "
+             "the child named '%s', needed to be read-only but was not. "
+             "The cap is being passed in a read slot (ro_uri), or was retrieved "
+             "from a read slot as an unknown cap." % name.encode("utf-8"))
+        return (t, http.BAD_REQUEST)
     if f.check(WebError):
         return (f.value.text, f.value.code)
     if f.check(FileTooLargeError):
@@ -170,7 +251,13 @@ class MyExceptionHandler(appserver.DefaultExceptionHandler):
         req.finishRequest(False)
 
     def renderHTTP_exception(self, ctx, f):
-        text, code = humanize_failure(f)
+        try:
+            text, code = humanize_failure(f)
+        except:
+            log.msg("exception in humanize_failure")
+            log.msg("argument was %s" % (f,))
+            log.err()
+            text, code = str(f), None
         if code is not None:
             return self.simple(ctx, text, code)
         if f.check(server.UnsupportedMethod):
